@@ -29,6 +29,7 @@ gboolean grab = TRUE;
 gboolean sudo_mode = FALSE;
 gboolean message_changed = FALSE;
 gboolean prompt = FALSE;
+gboolean always_ask_pass = FALSE;
 
 struct option long_opts[] = {
     /*
@@ -48,6 +49,7 @@ struct option long_opts[] = {
     {"debug", no_argument, NULL, 'd'},
     {"sudo-mode", optional_argument, NULL, 'S'},
     {"prompt", optional_argument, NULL, 'P'},
+    {"always-ask-password", optional_argument, NULL, 'a'},
     {0, 0, 0, 0}
 };
 
@@ -72,7 +74,6 @@ typedef struct {
 
 static GList *fadeout_windows = NULL;
 
-/* Go for five seconds */
 #define FADE_DURATION 100.0
 
 int
@@ -330,6 +331,10 @@ help (gchar *cmdname)
   g_print (_(
 "GKsu version %s\n\n"
 "Usage: %s [-u <user>] [-k] [-l] <command>\n\n"
+"  --always-ask-password, -a\n"
+"    Do not try to check if there are other means of\n"
+"    obtaining the password we need running the command,\n"
+"    or even if we really need it, just ask for it.\n"
 "  --debug, -d\n"
 "    Print information on the screen that might be\n"
 "    useful for diagnosing and/or solving problems.\n"
@@ -645,6 +650,11 @@ read_config_file_options (GksuContext *context)
 	  if (!strcasecmp ("yes", value))
 	    prompt = TRUE;
 	}
+      else if (!strcmp ("always-ask-password", key))
+	{
+	  if (!strcasecmp ("yes", value))
+	    always_ask_pass = TRUE;
+	}
 
       g_strfreev (tmp);
     }
@@ -714,7 +724,7 @@ main (int argc, char **argv)
 
   context = gksu_context_new ();
   read_config_file_options (context);
-  while ((c = getopt_long(newargc, newargv, "?hu:lpm:kt:i:gdsS::P::", long_opts, NULL))
+  while ((c = getopt_long(newargc, newargv, "?hu:lpm:kt:i:gdsS::P::a", long_opts, NULL))
 	 != EOF)
     {
       switch (c)
@@ -809,6 +819,8 @@ main (int argc, char **argv)
 	    }
 
 	  break;
+	case 'a':
+	  always_ask_pass = TRUE;
 	}
     }
 
@@ -828,7 +840,9 @@ main (int argc, char **argv)
 						"the eavesdropping of your password by a a malicious\n"
 						"application while you type it."));
       
-      if (gtk_dialog_run (GTK_DIALOG(d)) == GTK_RESPONSE_YES)
+      if (gtk_dialog_run (GTK_DIALOG(d)) == GTK_RESPONSE_NO)
+	grab = FALSE;
+      else
 	grab = TRUE;
       
       gtk_widget_destroy (d);
@@ -923,8 +937,7 @@ main (int argc, char **argv)
   if (sudo_mode)
     {
       /* test if we can run without a password */
-      gksu_context_sudo_run (context, &error);
-      if (error)
+      if (gksu_context_sudo_try_need_password (context))
 	{
 	  if (!message_changed)
 	    {
@@ -960,54 +973,60 @@ main (int argc, char **argv)
 	  }
 
 	  gksu_context_set_password (context, password);
-      
-	  error = NULL;
-	  gksu_context_sudo_run (context, &error);
-	  if (error)
-	    gk_dialog (GTK_MESSAGE_ERROR, 
-		       _("Failed to run %s as user %s:\n %s"),
-		       gksu_context_get_command (context),
-		       gksu_context_get_user (context),
-		       error->message);
+      	}
+
+      error = NULL;
+      gksu_context_sudo_run (context, &error);
+      if (error)
+	{
+	  gk_dialog (GTK_MESSAGE_ERROR, 
+		     _("Failed to run %s as user %s:\n %s"),
+		     gksu_context_get_command (context),
+		     gksu_context_get_user (context),
+		     error->message);
 	  return 3;
 	}
     }
   else
     {
-      if (!message_changed)
+      if (always_ask_pass || gksu_context_try_need_password (context))
 	{
-	  gchar *msg = 
-	    g_strdup_printf (_("<b>Please enter %s's password\n"
-			       "to run %s.</b>"),
-			     gksu_context_get_user (context),
-			     gksu_context_get_command (context));
+	  if (!message_changed)
+	    {
+	      gchar *msg = 
+		g_strdup_printf (_("<b>To run the program \"%s\" you need to "
+				   "enter the %s password</b>"),
+				 gksu_context_get_command (context),
+				 gksu_context_get_user (context));
 	  
-	  gksuui_dialog_set_message (GKSUUI_DIALOG(dialog), msg);
-	  g_free (msg);
+	      gksuui_dialog_set_message (GKSUUI_DIALOG(dialog), msg);
+	      g_free (msg);
+	    }
+
+	  int lock = 0;
+	  if (grab)
+	    lock = grab_keyboard_and_mouse (dialog);
+	  retvalue = gtk_dialog_run (GTK_DIALOG(dialog));
+	  gtk_widget_hide (dialog);
+	  if (grab)
+	    ungrab_keyboard_and_mouse (lock);
+
+	  while (gtk_events_pending ())
+	    gtk_main_iteration ();
+
+	  if (retvalue != GTK_RESPONSE_OK)
+	    return 2;
+
+	  {
+	    gchar *tmp;
+	    tmp = gksuui_dialog_get_password (GKSUUI_DIALOG(dialog));
+	    password = g_locale_from_utf8 (tmp, strlen (tmp), NULL, NULL, NULL);
+	    g_free (tmp);
+	  }
+
+	  gksu_context_set_password (context, password);
 	}
 
-      int lock = 0;
-      if (grab)
-	 lock = grab_keyboard_and_mouse (dialog);
-      retvalue = gtk_dialog_run (GTK_DIALOG(dialog));
-      gtk_widget_hide (dialog);
-      if (grab)
-	ungrab_keyboard_and_mouse (lock);
-
-      while (gtk_events_pending ())
-	gtk_main_iteration ();
-
-      if (retvalue != GTK_RESPONSE_OK)
-	  return 2;
-
-      {
-	gchar *tmp;
-	tmp = gksuui_dialog_get_password (GKSUUI_DIALOG(dialog));
-	password = g_locale_from_utf8 (tmp, strlen (tmp), NULL, NULL, NULL);
-	g_free (tmp);
-      }
-
-      gksu_context_set_password (context, password);
       gksu_context_run (context, &error);
 
       if (error) {
